package com.cn.tianji.util;

import com.cn.tianji.common.StringUtils;

import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.*;

public class CommonUtil {
    private static final SimpleDateFormat dateFormat = new SimpleDateFormat("EEE MMM dd HH:mm:ss zzz yyyy", Locale.ENGLISH);
    /**
     * 根据字段名称获取实体类该字段的数据
     *
     * @param obj 实体类对象
     * @param fieldName 字段名称
     * @return 字段的值，如果字段不存在或无法访问则返回null
     */
    public static Object getFieldValue(Object obj, String fieldName) {
        if (obj == null || fieldName == null || fieldName.isEmpty()) {
            return null;
        }

        Class<?> clazz = obj.getClass();
        try {
            // 获取字段对象，设置为可访问（即使它是私有的）
            Field field = clazz.getDeclaredField(fieldName);
            field.setAccessible(true);
            // 获取字段的值
            return field.get(obj);
        } catch (NoSuchFieldException e) {
            // 如果字段不存在
            System.err.println("Field not found: " + fieldName);
            return null;
        } catch (IllegalAccessException e) {
            // 如果字段无法访问（理论上不应该发生，因为我们已经设置了setAccessible(true)）
            System.err.println("Field access exception: " + fieldName);
            return null;
        }
    }

    /**
     * Object 转 BigDecimal
     * @param obj
     * @return
     */
    public static BigDecimal convertToBigDecimal(Object obj) {
        if (obj == null) {
            return null;  // 或者你可以抛出一个异常，根据你的需求
        }

        try {
            // 假设 obj 是一个字符串或其他可以转换为 BigDecimal 的类型
            // 这里使用 String 作为示例，因为 String 是常见的输入类型
            if (obj instanceof String) {
                return new BigDecimal((String) obj);
            } else if (obj instanceof BigDecimal) {
                return (BigDecimal) obj;
            } else if (obj instanceof Number) {
                return BigDecimal.valueOf(((Number) obj).doubleValue());
            } else {
                throw new IllegalArgumentException("Unsupported object type: " + obj.getClass().getName());
            }
        } catch (NumberFormatException e) {
            throw new IllegalArgumentException("Cannot convert object to BigDecimal: " + obj, e);
        }
    }

    /**
     * 将对象转换为字符串，如果对象为null则返回空字符串
     *
     * @param obj 要转换的对象
     * @return 转换后的字符串，如果对象为null则返回空字符串
     */
    public static String convertToString(Object obj) {
        if (obj == null) {
            return "";
        } else {
            return obj.toString();
        }
    }

    public static Date convertToDate(Object obj) {
        // 判断对象是否为空
        if (obj == null) {
            return null;
        }

        // 尝试将对象转换为字符串
        String dateStr = obj.toString();

        try {
            // 尝试解析字符串为日期
            return dateFormat.parse(dateStr);
        } catch (ParseException e) {
            // 如果解析失败，返回 null
            return null;
        }
    }

    public static BigDecimal convertStringToBigDecimal(String str) throws ParseException {
        boolean empty = StringUtils.isEmpty(str);
        if(empty){
            return BigDecimal.valueOf(-999.25);
        }
        // 定义DecimalFormat以确保字符串为有效的数字格式
        DecimalFormat decimalFormat = new DecimalFormat("#.##########");
        //启用对BigDecimal的解析
        decimalFormat.setParseBigDecimal(true);

        // 使用DecimalFormat将字符串解析为BigDecimal
        BigDecimal bigDecimal = (BigDecimal) decimalFormat.parse(str);

        // 设置比例和舍入模式
        bigDecimal = bigDecimal.setScale(2, RoundingMode.HALF_UP);
        return bigDecimal;
    }

    /**
     * 根据分钟数 高于30分钟则加分钟数满足下一整点时间 低于30分钟 只减去当前分钟数满足当前整点时间
     * @param currentTime
     * @return
     */
    public static LocalTime adjustTimeToNearestHour(LocalTime currentTime) {
        int minute = currentTime.getMinute();

        if (minute >= 30) {
            // 如果分钟数 >= 30，则加到下一个整点
            LocalTime nextHour = currentTime.plusHours(1);
            return nextHour.truncatedTo(ChronoUnit.HOURS);
        } else {
            // 如果分钟数 < 30，则减到当前整点
            return currentTime.truncatedTo(ChronoUnit.HOURS);
        }
    }


    /**
     * 日期字符串减去天数返回减去天数的日期
     * @param dateString
     * @param daysToSubtract
     * @return
     */
    public static String subtractDaysFromDate(String dateString, int daysToSubtract) {
        // Define the date format
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");

        // Parse the input date string to LocalDate
        LocalDate date = LocalDate.parse(dateString, formatter);

        // Subtract the specified number of days
        LocalDate resultDate = date.minus(daysToSubtract, ChronoUnit.DAYS);

        // Format the result date back to string
        return resultDate.format(formatter);
    }


    public static  String timeSub(String time,long minutesToSubtract){
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        // 减去的分钟数
//        long minutesToSubtract = 5;

        // 解析字符串为LocalDateTime对象
        LocalDateTime dateTime = LocalDateTime.parse(time, formatter);

        // 减去指定的分钟数
        LocalDateTime newDateTime = dateTime.minusMinutes(minutesToSubtract);

        // 格式化并输出新的日期时间
        String newDateTimeStr = newDateTime.format(formatter);
        return newDateTimeStr;
    }

    /**
     * Objec转 List<Map<String, Object>>
     * @param objects
     * @param <T>
     * @return
     * @throws IllegalAccessException
     */
    public static <T> List<Map<String, Object>> convertToListOfMaps(List<T> objects) throws IllegalAccessException {
        List<Map<String, Object>> result = new ArrayList<>();
        for (T object : objects) {
            result.add(convertToMap(object));
        }
        return result;
    }

    public static <T> Map<String, Object> convertToMap(T object) throws IllegalAccessException {
        Map<String, Object> result = new HashMap<>();
        if (object == null) {
            return result;
        }
        Class<?> clazz = object.getClass();
        Field[] fields = clazz.getDeclaredFields();
        for (Field field : fields) {
            field.setAccessible(true); // To access private fields
            result.put(field.getName(), field.get(object));
        }
        return result;
    }

    public static <T> List<T> castList(Object obj, Class<T> clazz)
    {
        List<T> result = new ArrayList<T>();
        if(obj instanceof List<?>)
        {
            for (Object o : (List<?>) obj)
            {
                result.add(clazz.cast(o));
            }
            return result;
        }
        return null;
    }
}
