package com.cn.tianji.service.impl;

import com.cn.tianji.common.CommonUtils;
import com.cn.tianji.service.ConditionDeterminationService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;

/**
 * 工况动作判定条件
 * @author Zhao RenJi
 */
@Service
@Slf4j
public class ConditionDeterminationServiceImpl  implements ConditionDeterminationService {
    /**
     *
     * 判断是否:在井底
     *
     * 井深-钻头深度<=在井底判定值
     * @param depth 井深
     * @param bitDepth 钻头深度
     * @param threshold 阈值
     * @return  true 在井底 false 未在井底
     */
    @Override
    public Boolean isOnBottom(BigDecimal depth, BigDecimal bitDepth, BigDecimal threshold){
        //井深减钻头深度
        BigDecimal depthSub = depth.subtract(bitDepth);
        //阈值比较结果
        int thresholdCom = depthSub.compareTo(threshold);
        return  thresholdCom<=0;
    }

    /**
     * 判断是否:到达地面
     * 10s内钻头平均深度<在地面判定值
     * @param bitDepthList 10秒内钻头深度数组
     * @param threshold 阈值
     * @return true 到达地面 false 未到达地面
     */
    @Override
    public Boolean isApproachingSurface(List<BigDecimal> bitDepthList, BigDecimal threshold){
        //求钻头深度平均值
        BigDecimal averageBitDepth = CommonUtils.calculateAverage(bitDepthList);
        int res = averageBitDepth.compareTo(threshold);
        return res<0;
    }
    /**
     * 判断是否:离开井底
     * 10s内井深减钻头深度平均值>在地面判定值
     * @param bitDepthList 钻头深度数组
     * @param depthList     井深数组
     * @param threshold 阈值
     * @return true 离开井底 false 未离开井底
     */
    @Override
    public Boolean isFarBottom(List<BigDecimal> bitDepthList,List<BigDecimal> depthList,BigDecimal threshold){

        if(bitDepthList.size()==0){
            log.info("钻头位置数据长度为0");
           return true;
        }
        if(bitDepthList.size()==0){
            log.info("井深数组长度为0");
            return true;
        }
        BigDecimal bitDepthSum = bitDepthList.stream().reduce(BigDecimal.ZERO,BigDecimal::add);
        BigDecimal depthSum = depthList.stream().reduce(BigDecimal.ZERO, BigDecimal::add);
        BigDecimal depthSubtract = depthSum.subtract(bitDepthSum);
        BigDecimal averageBitDepth= depthSubtract.divide(BigDecimal.valueOf(bitDepthList.size()),2, RoundingMode.HALF_UP);
//        log.info("是否离开井底:井深均值{};阈值:{}",averageBitDepth,threshold);
        int thresholdCom = averageBitDepth.compareTo(threshold);
        return thresholdCom>0;
    }

    /**
     * 判断是否:停转
     * 转速<停转判定值
     * @param rpm 转盘转速
     * @param threshold 阈值
     * @return true 停转 false 未停转
     */
    @Override
    public Boolean isRotationOff(BigDecimal rpm,BigDecimal threshold){
        int thresholdCom =  rpm.compareTo(threshold);
        return thresholdCom<=0;
    }

    /**
     * 判断是否:停泵
     * 出口流量<出口流量设定值(泵冲)
     * @param pump1 一号泵冲
     * @param pump2 二号泵冲
     * @param pump3 三号泵冲
     * @param threshold 阈值
     * @return true 停泵 false 未停泵
     */
    @Override
    public Boolean isNoFlow(BigDecimal pump1,BigDecimal pump2,BigDecimal pump3,BigDecimal threshold){
        BigDecimal sumPump =calSumPump(pump1,pump2,pump3);
        int thresholdCom =sumPump.compareTo(threshold);
        return thresholdCom<=0;
    }

    /**
     * 判断是否:立压为0
     * 立压<立压设定值
     * @param spp 立管压力
     * @param threshold 阈值
     * @return true 立压为0 false 立压不为0
     */
    @Override
    public Boolean isPressureZero(BigDecimal spp,BigDecimal threshold){
        int thresholdCom =spp.compareTo(threshold);
        return thresholdCom<=0;
    }

    /**
     * 判断是否:游车静止
     * 游车平均速度=0（大钩上升下降速度）
     * @param nowHkh 当前点大钩高度
     * @param perHkh 上一点大钩高度
     * @param timeSub 时间差
     * @param threshold 阈值
     * @return true 游车静止 false 游车未静止
     */
    @Override
    public Boolean isBlockldle(BigDecimal nowHkh,BigDecimal perHkh,BigDecimal timeSub,BigDecimal threshold){
        BigDecimal hkhSubtract = nowHkh.subtract(perHkh);
        BigDecimal averageSpeed = hkhSubtract.divide(timeSub, 2, RoundingMode.HALF_UP);
        int thresholdCom =averageSpeed.compareTo(threshold);
        return thresholdCom==0;
    }

    /**
     * 判断是否:钻头上行
     * 钻头平均速度<钻头速度设定值
     * @param nowBitdepth 当前点钻头位置
     * @param perBitdepth 上一点钻头位置
     * @param timeSub 时间差
     * @param threshold 阈值
     * @return true 钻头上行 false 钻头下行
     */
    @Override
    public Boolean isBitMovingUp(BigDecimal nowBitdepth,BigDecimal perBitdepth,BigDecimal timeSub,BigDecimal threshold){
        BigDecimal hkhSubtract = nowBitdepth.subtract(perBitdepth);
        BigDecimal averageSpeed = hkhSubtract.divide(timeSub, 2, RoundingMode.HALF_UP);
        int thresholdCom =averageSpeed.compareTo(threshold);
        return thresholdCom>=0;
    }

    /**
     * 判断是否:游车上行
     * 游车平均速度>游车速度设定
     * @param nowHkh 当前点大钩高度
     * @param perHkh 上一点大钩高度
     * @param timeSub 时间差
     * @param threshold 阈值
     * @return true 游车上行 false 游车下行
     */
    @Override
    public Boolean isBlockMovingUp(BigDecimal nowHkh,BigDecimal perHkh,BigDecimal timeSub,BigDecimal threshold){
        BigDecimal hkhSubtract = nowHkh.subtract(perHkh);
        BigDecimal averageSpeed = hkhSubtract.divide(timeSub, 2, RoundingMode.HALF_UP);
        int thresholdCom =averageSpeed.compareTo(threshold);
        return thresholdCom>0;
    }

    /**
     * 判断是否:坐卡
     * 钩载<钩载设定
     * @param wob 当前点钩载
     * @param threshold 阈值
     * @return true 坐卡 false 未坐卡
     */
    @Override
    public Boolean isInSlips(BigDecimal wob,BigDecimal threshold){
        int thresholdCom =wob.compareTo(threshold);
        return thresholdCom<0;
    }

    /**
     * 计算总泵冲
     * @param pumpB1 一号泵冲
     * @param pumpB2 二号泵冲
     * @param pumpB3 三号泵冲
     * @return 总泵冲
     */
    public BigDecimal calSumPump(BigDecimal pumpB1, BigDecimal pumpB2, BigDecimal pumpB3) {
        BigDecimal zero = BigDecimal.ZERO;
        BigDecimal res = BigDecimal.ZERO;
        //判断三个泵冲是否有参数值;负值为无参数
        if (pumpB1.compareTo(zero) >= 0 && pumpB2.compareTo(zero) >= 0 && pumpB3.compareTo(zero) >= 0) {
            res = pumpB1.add(pumpB2).add(pumpB3);
        } else {
            if (pumpB1.compareTo(zero) >= 0 && pumpB2.compareTo(zero) >= 0) {
                res = pumpB1.add(pumpB2);
            } else if (pumpB1.compareTo(zero) >= 0 && pumpB3.compareTo(zero) >= 0) {
                res = pumpB1.add(pumpB3);
            } else if (pumpB2.compareTo(zero) >= 0 && pumpB3.compareTo(zero) >= 0) {
                res = pumpB2.add(pumpB2);
            } else {
                if (pumpB1.compareTo(zero) >= 0) {
                    res = pumpB1;
                } else if (pumpB2.compareTo(zero) >= 0) {
                    res = pumpB2;
                } else if (pumpB3.compareTo(zero) >= 0) {
                    res = pumpB3;
                }
            }
        }
        return res;
    }
}
